ST.virginia = function () {
    function get_rect_tooltip(passing_index, score_labels) {
        return function(d, i) {
            if (i<passing_index) {
                return score_labels[i];
            } else {
                var n = score_labels.length,
                    passing = score_labels.slice(passing_index, n).reverse();
                return passing[i-passing_index];
            }
        };
    };
    function get_ticks(max, count) {
        var delta = max/count;
        result = [];
        for (i=0; i<=count; i++) {
            result.push(i*delta);
            result.push(i*-delta);
        }
        return result;
    };
    function get_tick_format(total, row_number, row_step) {
        return function(d, i) {
            var value = Math.round(Math.abs(d*100/total)),
                show = row_number % row_step == 0;
            if (show && (value > 0)) {
                return value;
            }
        };
    };
    function transform_data(data, total, passing_index) {
        var result = [];
        for (var i=0;i<data.length;i++) {
            d = data[i];
            if (i<passing_index) {
                result.push(-d3.sum(data.slice(i, passing_index)));
            } else {
                result.push(d3.sum(data.slice(passing_index, i+1)));
            }
        }
        var not_passing = result.slice(0, passing_index);
        var passing = result.slice(passing_index, data.length).reverse();
        return not_passing.concat(passing);
    }
    function get_rect_class(passing_index) {
        return function(d, i) {
            return i>=passing_index?'passing':undefined;
        };
    }
    function get_chart_class(data, total, passing_index, cutoff) {
        return function(d, i) {
            var max = d3.max(data.slice(passing_index, data.length));
            if ((max/total) >= (cutoff/100)) {
                return 'chart above-cutoff';
            } else {
                return 'chart';
            }
        };
    };
    function get_passing_colors() {
        return d3.interpolateRgb('#6B3A89', '#D1C4DA');
    };
    function get_not_passing_colors() {
        return d3.interpolateRgb('#F68E4D', '#FCD6C5');
    };
    function get_rect_color(data, total, passing_index) {
        var passing_colors = get_passing_colors(),
            not_passing_colors = get_not_passing_colors();
        return function(d, i) {
            if (i>=passing_index) {
                j = i-passing_index;
                passing = data.slice(passing_index, data.length);
                if (passing.length > 1) {
                    return passing_colors(j/(passing.length-1));
                } else {
                    return passing_colors(j/passing.length);
                }
            } else {
                not_passing = data.slice(0, passing_index);
                if (not_passing.length > 1) {
                    return not_passing_colors(i/(not_passing.length-1));
                } else {
                    return not_passing_colors(i/not_passing.length);
                }
            }
        };
    };
    function get_filter_fields() {
        return $('.sidebar .report-filter');
    };
    function get_filter_field_values(fields) {
        var result = [],
            field;
        fields.each(function(index, element) {
            field = $(element);
            if (field.is('select') || field.is('input:checked')) {
                result.push({
                    name: field.attr('name'),
                    value: field.val()
                });
            }
        });
        return result;
    };
    function append_filter_fields(container_id) {
        var container = $(ST.dialogs.jquery_id(container_id)),
            form = container.find('form'),
            filter_fields = get_filter_fields(),
            filter_field_values = get_filter_field_values(filter_fields),
            input_field;
        $.each(filter_field_values, function(index, field) {
            input_field = $('<input type="hidden" />')
                .attr('name', field.name)
                .attr('value', field.value);
            form.append(input_field);
        });
    }
    return {
        get_filter_fields: get_filter_fields,
        get_filter_field_values: get_filter_field_values,
        build_chart: function (container_id, data, total, passing_index,
                               score_labels, cutoff) {
            var transformed = transform_data(data, total, passing_index),
                h = 12,
                w = 220,
                chart_width = w-20,
                container = $(container_id),
                row = container.closest('tr'),
                row_step = 1,
                xTickCount = 5,
                xTickSize = 2,
                xTickPadding = 2,
                xTickValues = get_ticks(total, xTickCount),
                x = d3.scale.linear()
                    .domain([-total, total])
                    .range([0, chart_width]),
                chart = d3.select(container_id)
                    .append('svg')
                    .attr('class', get_chart_class(transformed, total,
                                                   passing_index, cutoff))
                    .attr('height', h)
                    .attr('width', w)
                    .append('g')
                    .attr('transform', 'translate(10,0)'),
                x_axis = d3.svg.axis()
                    .scale(x)
                    .orient('top')
                    .tickFormat(get_tick_format(total, row.index(),
                                                row_step))
                    .tickPadding(xTickPadding)
                    .tickSize(xTickSize)
                    .tickValues(xTickValues);
            chart.append('g')
                .attr('class', 'bg-chart')
                .append('rect')
                .attr('width', chart_width)
                .attr('height', h);
            chart.append('g')
                .attr('class', 'bg-cutoff')
                .append('rect')
                .attr('width', chart_width*((cutoff/200) + 0.5))
                .attr('height', h);
            chart.selectAll('rect')
                .data(transformed)
                .enter().append('rect')
                .attr('class', get_rect_class(passing_index))
                .attr('x',
                      function(d, i) {
                          return x(Math.min(0, d));
                 })
                .attr('y', 0)
                .attr('width',
                      function(d, i) {
                          return Math.abs(x(d) - x(0));
                 })
                .attr('height', h-0.5)
                .attr('fill', get_rect_color(data, total, passing_index))
                .append('title').text(
                    get_rect_tooltip(passing_index, score_labels));
            chart.append('g')
                .attr('class', 'x-axis axis')
                .call(x_axis)
                .attr('transform', 'translate(0,'+(h-0.25)+')');
            chart.append('g')
                .attr('class', 'y-axis axis')
                .append('line')
                .attr('x1', x(0))
                .attr('x2', x(0))
                .attr('y1', 0)
                .attr('y2', h);
            $('.above-cutoff').closest('tr').addClass('above-cutoff');
        },
        on_section_report_sort: function(container_id, column_name,
                                         sort_on_name) {
            append_filter_fields(container_id);
            ST.table.on_form_sort(container_id, column_name, sort_on_name);
        },
        build_color_cell: function(container_id, score) {
            var passing_colors = get_passing_colors(),
                not_passing_colors = get_not_passing_colors(),
                container = $(container_id);
            color = $('<span class="color">&nbsp;</span>');
            title = $('<span class="title">'+score.title+'</span>');
            if (score.is_passing) {
                colors = passing_colors;
            } else {
                colors = not_passing_colors;
            }
            color.css('backgroundColor', colors(score.color_weight));
            container.append(color, title);
        }
    };
}();
