#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Virginia Initialization
"""

from schooltool.app.app import InitBase, StartUpBase
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.interfaces import IDemographicsFields

from schooltool.virginia import VirginiaMessage as _


def setUpDemographics(app):
    dfs = IDemographicsFields(app)

    # custom ethnicity choices
    if 'ethnicity' not in dfs:
        dfs['ethnicity'] = EnumFieldDescription(
            'ethnicity', _('Ethnicity'))
    ethnicity = dfs['ethnicity']
    if not ethnicity.items or ethnicity.items[0] != u'Asian':
        ethnicity.items = [
            u'Asian',
            u'Black',
            u'Hispanic',
            u'American Indian',
            u'Multiracial',
            u'Pacific Island',
            u'Unspecified',
            u'White',
            ]

    # student fields
    if 'grade_level' not in dfs:
        dfs['grade_level'] = TextFieldDescription(
            'grade_level', _('Grade_Level'),
            limit_keys=['students'])
    if 'state_report_id' not in dfs:
        dfs['state_report_id'] = TextFieldDescription(
            'state_report_id', _('State Report ID'),
            limit_keys=['students'])


class VirginiaAppStartup(StartUpBase):
    def __call__(self):
        setUpDemographics(self.app)


class VirginiaInit(InitBase):
    def __call__(self):
        setUpDemographics(self.app)

